@@ -1,4 +1,5 @@
-/* Copyright (C) 1994-1998, 2000-2011 Free Software Foundation, Inc.
+/* Copyright (C) 1994-1998, 2000-2011, 2013, 2014, 2019 Free Software Foundation, Inc.
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
  * as published by the Free Software Foundation; either version 3 of
@@ -33,7 +34,6 @@
 #include "libguile/eval.h"
 #include "libguile/feature.h"
 #include "libguile/load.h"
-#include "libguile/private-gc.h" /* scm_getenv_int */
 #include "libguile/read.h"
 #include "libguile/script.h"
 #include "libguile/strings.h"
@@ -46,9 +46,7 @@
 #include <string.h>
 #endif
 
-#ifdef HAVE_UNISTD_H
 #include <unistd.h>		/* for X_OK define */
-#endif
 
 #ifdef HAVE_IO_H
 #include <io.h>
@@ -75,7 +73,7 @@ scm_cat_path (char *str1, const char *str2, long n)
   if (!str1)
     return 0L;
   str1[0] = 0;
-  strncat (str1, str2, n);
+  strncat (str1, str2, n + 1);
   return str1;
 }
 
@@ -221,6 +219,21 @@ script_get_backslash (FILE *f)
 }
 #undef FUNC_NAME
 
+/*
+ * Like `realloc', but free memory on failure;
+ * unlike `scm_realloc', return NULL, not aborts.
+*/
+static void*
+realloc0 (void *ptr, size_t size)
+{
+  void *new_ptr = realloc (ptr, size);
+  if (!new_ptr)
+    {
+      free (ptr);
+    }
+  return new_ptr;
+}
+
 
 static char *
 script_read_arg (FILE *f)
@@ -246,7 +259,7 @@ script_read_arg (FILE *f)
 	  if (len >= size)
 	    {
 	      size = (size + 1) * 2;
-	      buf = realloc (buf, size);
+	      buf = realloc0 (buf, size);
 	      if (! buf)
 		return 0;
 	    }
@@ -329,9 +342,9 @@ scm_get_meta_args (int argc, char **argv)
 	found_args:
           /* FIXME: we leak the result of calling script_read_arg.  */
 	  while ((narg = script_read_arg (f)))
-	    if (!(nargv = (char **) realloc (nargv,
+	    if (!(nargv = (char **) realloc0 (nargv,
 					     (1 + ++nargc) * sizeof (char *))))
-	        return 0L;
+	      return 0L;
 	    else
 	      nargv[nargi++] = narg;
 	  fclose (f);
